@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2015 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/apache.
@@ -166,7 +166,10 @@ class Ingo_Script_Sieve_Action_Vacation extends Ingo_Script_Sieve_Action
      */
     protected function _vacationCode()
     {
-        $code = 'vacation :days ' . $this->_vars['days'] . ' ';
+        $code = 'vacation ';
+        if (!empty($this->_vars['days'])) {
+            $code .= ':days ' . $this->_vars['days'] . ' ';
+        }
         $addresses = $this->_vars['addresses'];
         $stringlist = '';
         if (count($addresses) > 1) {
